/*
*  Copyright (C) 2007 Anand Sengupta, Thomas Cokelaer
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with with program; see the file COPYING. If not, write to the
*  Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA  02111-1307  USA
*/

/*  <lalVerbatim file="LALGenerateInspiralWaveformCV">
Author: Sathyaprakash, B. S., Cokelaer T.
$Id: GenerateInspiralWaveform.c,v 1.9 2008/10/08 22:17:02 spxcar Exp $
</lalVerbatim>  */

/*  <lalLaTeX>

\subsection{Test program \texttt{LALGenerateInspiralWaveform.c}}
Test routine for wave generation codes. 

To get some help just type the name of the executable and the option --h

Basically, you can provide all the arguments from the InspiralTemplate structure such as
--approximant, --order ....

\vfill{\footnotesize\input{LALGenerateInspiralWaveformCV}}

</lalLaTeX> */


#define LALGENERATEINSPIRALWAVEFORMC_ENORM 0
#define LALGENERATEINSPIRALWAVEFORMC_ESUB  1
#define LALGENERATEINSPIRALWAVEFORMC_EARG  2
#define LALGENERATEINSPIRALWAVEFORMC_EVAL  3
#define LALGENERATEINSPIRALWAVEFORMC_EFILE 4
#define LALGENERATEINSPIRALWAVEFORMC_EMEM  5

#define LALGENERATEINSPIRALWAVEFORMC_MSGENORM "Normal exit"
#define LALGENERATEINSPIRALWAVEFORMC_MSGESUB  "Subroutine failed"
#define LALGENERATEINSPIRALWAVEFORMC_MSGEARG  "Error parsing arguments"
#define LALGENERATEINSPIRALWAVEFORMC_MSGEVAL  "Input argument out of valid range"
#define LALGENERATEINSPIRALWAVEFORMC_MSGEFILE "Could not open file"
#define LALGENERATEINSPIRALWAVEFORMC_MSGEMEM  "Out of memory"

#include <stdio.h>
#include <lal/LALInspiral.h>
#include <lal/RealFFT.h>
#include <lal/AVFactories.h>

NRCSID( LALGENERATEINSPIRALWAVEFORMC, "$Id: GenerateInspiralWaveform.c,v 1.9 2008/10/08 22:17:02 spxcar Exp $" );

INT4 lalDebugLevel=7;
#define ERROR( code, msg, statement )                                \
do                                                                   \
if ( lalDebugLevel & LALERROR )                                      \
{                                                                    \
  LALPrintError( "Error[0] %d: program %s, file %s, line %d, %s\n"   \
		 "        %s %s\n", (code), program, __FILE__,       \
		 __LINE__, LALGENERATEINSPIRALWAVEFORMC, statement ? statement :  \
                 "", (msg) );                                        \
}                                                                    \
while (0)

#define WARNING( statement )                                         \
do                                                                   \
if ( lalDebugLevel & LALWARNING )                                    \
{                                                                    \
  LALPrintError( "Warning[0]: program %s, file %s, line %d, %s\n"    \
		 "        %s\n", program, __FILE__, __LINE__,        \
		 LALGENERATEINSPIRALWAVEFORMC, (statement) );                         \
}                                                                    \
while (0)

#define INFO( statement )                                            \
do                                                                   \
if ( lalDebugLevel & LALINFO )                                       \
{                                                                    \
  LALPrintError( "Info[0]: program %s, file %s, line %d, %s\n"       \
		 "        %s\n", program, __FILE__, __LINE__,        \
		 LALGENERATEINSPIRALWAVEFORMC, (statement) );                         \
}                                                                    \
while (0)

#define SUB( func, statusptr )                                       \
do                                                                   \
if ( (func), (statusptr)->statusCode )                               \
{                                                                    \
  ERROR( LALGENERATEINSPIRALWAVEFORMC_ESUB, LALGENERATEINSPIRALWAVEFORMC_MSGESUB,                      \
         "Function call \"" #func "\" failed:" );                    \
  exit( LALGENERATEINSPIRALWAVEFORMC_ESUB );                                          \
}                                                                    \
while (0)

typedef struct{
  INT4 PrintParameters;
} OtherParamIn;

char *program;

void printf_timeseries (UINT4 n, REAL4 *signal, REAL8 delta, REAL8 t0) ;
void printf_timeseries2 (UINT4 n, REAL4 *signal1, REAL4 *signal2, REAL8 delta, REAL8 t0) ;
void ParseParameters(UINT4 argc, CHAR **argv, OtherParamIn *otherIn);
void LALGenerateInspiralWaveformHelp(void);



/* --- Main part --- */
int main (int argc , char **argv) {
  REAL4Vector *signal1 = NULL; /*storing waveforms */
  REAL4Vector *signal2 = NULL; /*storing waveforms */
  static LALStatus status;
  InspiralTemplate params; /* the parameters */
  REAL8 dt;                /* some sampling */
  UINT4 n,i;
  InspiralInit paramsInit;

  RealFFTPlan *revp =NULL;

  COMPLEX8Vector *Signal1 =NULL;

  OtherParamIn otherIn; /* some extra parameters to parse*/

  program = *argv;

  /* ---  we start real computation here --- */
  otherIn.PrintParameters = 0; /* by default we don't print the parameters */
  ParseParameters(argc, argv, &otherIn);/*let's parse user parameters     */


  SUB( LALInspiralITStructureSetDefault(&status, &params),
       &status); /*  */ 
  
  SUB( LALInspiralITStructureParseParameters(&status, argc, argv, &params), 
       &status);/*parse user inspiral template parameters */

 
  SUB(  LALInspiralInit(&status, &params, &paramsInit), &status);
  /*  params.signalAmplitude =1;*/
 
  
  if (otherIn.PrintParameters){
    fprintf(stderr, "the inspiral structure (your parameters) before the call to the waveform generation:\n");
      SUB( LALInspiralITStructurePrint(&status, params),  &status); 
  }
     
  /* force those parameters */

  dt 	= 1./params.tSampling;
  n 	= paramsInit.nbins;   
  
  if (otherIn.PrintParameters)
    {
      fprintf(stderr, "#Testing Inspiral Signal Generation Codes:\n");
      fprintf(stderr, "#Signal length=%d, t0=%e, t2=%e, t3=%e\n", n, params.t0, params.t2, params.t3);  
      fprintf(stderr,"#size in bins %d\n",n);
      fprintf(stderr,"#size in seconds %f\n",params.tC);
    }
  
  SUB( LALSCreateVector(&status, &(signal1), n), &status);
  SUB( LALSCreateVector(&status, &(signal2), n), &status);
     
  params.ieta = 1;  /*should be zero or 1 ?? */
  /* */
  
  switch (params.approximant){		   
  case TaylorF1:
  case TaylorF2:
  case BCV:
  case BCVSpin:
    SUB( LALInspiralWave(&status, signal1, &params), &status);
    SUB( LALCreateReverseRealFFTPlan(&status, &revp, n, 0), &status);
        
    SUB(   LALCCreateVector(&status, &Signal1, n/2+1), &status);
    for (i=1; i<n/2; i++) 
      {
	Signal1->data[i].re = signal1->data[i];
	Signal1->data[i].im = signal1->data[n-i];
      }
    Signal1->data[0].re = 0.;
    Signal1->data[0].im = 0.;
    Signal1->data[n/2].re = 0.;
    Signal1->data[n/2].im = 0.;

    SUB( LALReverseRealFFT(&status, signal2, Signal1, revp), &status);
    SUB( LALCDestroyVector (&status, &Signal1), &status);
    SUB( LALDestroyRealFFTPlan (&status, &revp), &status);
    SUB( LALSDestroyVector(&status, &signal2), &status);
    SUB( LALSDestroyVector(&status, &signal1), &status);
    break;
  case SpinTaylorT3:
  case TaylorT1:
  case TaylorT2:
  case TaylorT3:
  case EOB:
  case EOBNR:
  case PadeT1:
  case SpinTaylor:
  case Eccentricity:
    /*
    SUB(LALInspiralWave(&status, signal2, &params), &status);
    */
    SUB(LALInspiralWaveTemplates(&status, signal1, signal2, &params), &status);
    if (status.statusCode == 0){
      printf_timeseries2(signal1->length, signal2->data, signal1->data, dt, params.startTime);   	  
      SUB( LALSDestroyVector(&status, &signal2), &status);
      SUB( LALSDestroyVector(&status, &signal1), &status);
    }
    else 
      {
	SUB( LALSDestroyVector(&status, &signal1), &status);
	SUB( LALSDestroyVector(&status, &signal2), &status);	
      }
    break;
  case PadeF1:
  default:
	    break;
  }

  fprintf(stderr, "fFinal = %f Hz tFinal = %f seconds\n" , params.fFinal, params.tC) ;
  
  if (otherIn.PrintParameters){
    fprintf(stderr, "the inspiral structure after the call to the waveform generation:\n");
      SUB( LALInspiralITStructurePrint(&status, params),  &status); 
  }
  
  LALCheckMemoryLeaks();
  return 0;
}





void 
ParseParameters(	UINT4 			argc, 
			CHAR 			**argv,
			OtherParamIn    	*otherIn)
{
  UINT4 		i = 1;
  
  while(i < argc)
    {
      if ( strcmp(argv[i],	"--verbose") 	== 0 ) {
	otherIn->PrintParameters = 1;
      }
      else if( strcmp(argv[i],	"--h") 	== 0 ) {
	LALGenerateInspiralWaveformHelp(); }
      else if( strcmp(argv[i],"-h") 	== 0 ) {
	LALGenerateInspiralWaveformHelp();
      }
      else if( strcmp(argv[i],"-help") 	== 0 ) {
	LALGenerateInspiralWaveformHelp();
      } 
      else if( strcmp(argv[i],"--help")	== 0 ) {
	LALGenerateInspiralWaveformHelp();
      }
      
      i++;
    }
  
}



void LALGenerateInspiralWaveformHelp(void)
{

  fprintf(stderr,"LALGenerateInspiralWaveform Help\n");
  fprintf(stderr, "-----------------------------------------------\n");
  fprintf(stderr, "--h for help\n");
  fprintf(stderr, "--verbose to print Inspiral Template parameters\n");
  fprintf(stderr, "-----------------------------------------------\n");
  LALInspiralITStructureHelp();

}



void printf_timeseries (UINT4 n, REAL4 *signal, REAL8 delta, REAL8 t0) 
{
  UINT4 i=0;
  FILE *outfile1;

  outfile1=fopen("wave1.dat","w");

  do 
     /* fprintf (outfile1,"%e %e\n", i*delta+t0, *(signal+i)  );
      */
     fprintf (outfile1,"%e\n", *(signal+i)  );
  while (n-++i); 

  fprintf(outfile1,"&\n");
  fclose(outfile1);
}



void printf_timeseries2 (UINT4 n, REAL4 *signal1, REAL4 *signal2, REAL8 delta, REAL8 t0) 
{
  UINT4 i=0;
  FILE *outfile1;

  outfile1=fopen("wave2.dat","w");

  do 
     /* fprintf (outfile1,"%e %e\n", i*delta+t0, *(signal+i)  );
      */
     fprintf (outfile1,"%e %e %e\n", i*delta, *(signal1+i), *(signal2+i)  );
  while (n-++i); 

  fclose(outfile1);
}

